package cz.mendelu.dnaAnalyser.analyse.cpg;

import cz.mendelu.dnaAnalyser.sequence.Sequence;
import cz.mendelu.dnaAnalyser.user.User;
import cz.mendelu.dnaAnalyser.utils.analyse.model.Analyse;
import cz.mendelu.dnaAnalyser.utils.model.Identifiable;
import lombok.*;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.Table;
import java.util.Date;
import java.util.Set;
import java.util.UUID;

@Data
@Entity
@ToString(callSuper = true)
@NoArgsConstructor
@EqualsAndHashCode(callSuper = true)
@Table(name = "cpg_analyses")
public class Cpg extends Analyse implements Identifiable {

    @Column(name = "min_window_size")
    private Integer minWindowSize;

    @Column(name = "min_gc_percentage")
    private Double minGcPercentage;

    @Column(name = "min_observed_to_expected_cpg")
    private Double minObservedToExpectedCpG;

    @Column(name = "min_island_merge_gap")
    private Integer minIslandMergeGap;

    @Column(name = "first_nucleotide")
    private Character firstNucleotide;

    @Column(name = "second_nucleotide")
    private Character secondNucleotide;

    @Builder
    public Cpg(UUID id, Date created, Sequence sequence, User owner, Set<String> tags, Date finished, Long resultCount,
               Integer minWindowSize, Double minGcPercentage, Double minObservedToExpectedCpG, Integer minIslandMergeGap,
               Character firstNucleotide, Character secondNucleotide) {
        super(id != null ? id : UUID.randomUUID(), created, sequence, owner, tags, finished, sequence.getName(), resultCount);
        this.minWindowSize = minWindowSize != null ? minWindowSize : 200;
        this.minGcPercentage = minGcPercentage != null ? minGcPercentage : 0.5;
        this.minObservedToExpectedCpG = minObservedToExpectedCpG != null ? minObservedToExpectedCpG : 0.6;
        this.minIslandMergeGap = minIslandMergeGap != null ? minIslandMergeGap : 100;
        this.firstNucleotide = firstNucleotide != null ? firstNucleotide : 'C';
        this.secondNucleotide = secondNucleotide != null ? secondNucleotide : 'G';
    }
}
