package cz.mendelu.dnaAnalyser.analyse.g4hunter.quadruplex;

import cz.mendelu.dnaAnalyser.utils.analyse.data.DataLocalisedMapper;
import org.apache.ibatis.annotations.*;
import org.apache.ibatis.cursor.Cursor;
import org.apache.ibatis.session.RowBounds;

import java.util.List;

public interface AggregateQuadruplexMapper extends DataLocalisedMapper<AggregateQuadruplex> {

    // language=SQL
    String TABLE = "CREATE TABLE AGGREGATE (" +
            "ID integer NOT NULL AUTO_INCREMENT, " +
            "POSITION integer, " +
            "LENGTH integer, " +
            "SCORE float, " +
            "ABS_SCORE float, " +
            "SEQUENCE varchar, " +
            "SUB_SCORE varchar, " +
            "PRIMARY KEY (ID))";

    // language=SQL
    @Select("SELECT count(*) FROM AGGREGATE")
    int countAll();

    // language=SQL
    @Select("SELECT * FROM AGGREGATE")
    @Results(value = {
            @Result(property = "id", column = "ID"),
            @Result(property = "position", column = "POSITION"),
            @Result(property = "length", column = "LENGTH"),
            @Result(property = "score", column = "SCORE"),
            @Result(property = "absScore", column = "ABS_SCORE"),
            @Result(property = "sequence", column = "SEQUENCE"),
            @Result(property = "subScore", column = "SUB_SCORE")
    })
    Cursor<AggregateQuadruplex> getCursor();

    // language=SQL
    @Select("SELECT * FROM AGGREGATE")
    @Results(value = {
            @Result(property = "id", column = "ID"),
            @Result(property = "position", column = "POSITION"),
            @Result(property = "length", column = "LENGTH"),
            @Result(property = "score", column = "SCORE"),
            @Result(property = "absScore", column = "ABS_SCORE"),
            @Result(property = "sequence", column = "SEQUENCE"),
            @Result(property = "subScore", column = "SUB_SCORE")
    })
    List<AggregateQuadruplex> getAll(RowBounds rowBounds);

    // language=SQL
    @Select("SELECT count(*) FROM AGGREGATE WHERE POSITION >= #{startPosition} AND POSITION < #{endPosition}")
    int countAllBetweenPosition(
            @Param("startPosition") int startPosition,
            @Param("endPosition") int endPosition);

    // language=SQL
    @Select("SELECT * FROM AGGREGATE")
    @Results(value = {
            @Result(property = "id", column = "ID"),
            @Result(property = "position", column = "POSITION"),
            @Result(property = "length", column = "LENGTH"),
            @Result(property = "score", column = "SCORE"),
            @Result(property = "absScore", column = "ABS_SCORE"),
            @Result(property = "sequence", column = "SEQUENCE"),
            @Result(property = "subScore", column = "SUB_SCORE")
    })
    List<AggregateQuadruplex> getAllItems();

    // language=SQL
    @Select("SELECT * FROM AGGREGATE WHERE POSITION >= #{startPosition} AND POSITION < #{endPosition} ORDER BY ${orderName} ASC")
    @Results(value = {
            @Result(property = "id", column = "ID"),
            @Result(property = "position", column = "POSITION"),
            @Result(property = "length", column = "LENGTH"),
            @Result(property = "score", column = "SCORE"),
            @Result(property = "absScore", column = "ABS_SCORE"),
            @Result(property = "sequence", column = "SEQUENCE"),
            @Result(property = "subScore", column = "SUB_SCORE")
    })
    List<AggregateQuadruplex> getAllBetweenPositionOrderAsc(
            @Param("startPosition") int startPosition,
            @Param("endPosition") int endPosition,
            @Param("orderName") String orderName,
            RowBounds rowBounds);

    // language=SQL
    @Select("SELECT * FROM AGGREGATE WHERE POSITION >= #{startPosition} AND POSITION < #{endPosition} ORDER BY ${orderName} DESC")
    @Results(value = {
            @Result(property = "id", column = "ID"),
            @Result(property = "position", column = "POSITION"),
            @Result(property = "length", column = "LENGTH"),
            @Result(property = "score", column = "SCORE"),
            @Result(property = "absScore", column = "ABS_SCORE"),
            @Result(property = "sequence", column = "SEQUENCE"),
            @Result(property = "subScore", column = "SUB_SCORE")
    })
    List<AggregateQuadruplex> getAllBetweenPositionOrderDesc(
            @Param("startPosition") int startPosition,
            @Param("endPosition") int endPosition,
            @Param("orderName") String orderName,
            RowBounds rowBounds);

    // language=SQL
    @Select("SELECT * FROM AGGREGATE WHERE ID = #{id}")
    @Results(value = {
            @Result(property = "id", column = "ID"),
            @Result(property = "position", column = "POSITION"),
            @Result(property = "length", column = "LENGTH"),
            @Result(property = "score", column = "SCORE"),
            @Result(property = "absScore", column = "ABS_SCORE"),
            @Result(property = "sequence", column = "SEQUENCE"),
            @Result(property = "subScore", column = "SUB_SCORE")
    })
    AggregateQuadruplex getById(int id);

    // language=SQL
    @Update("UPDATE AGGREGATE SET POSITION = #{position}, LENGTH = #{length}, SCORE = #{score}, ABS_SCORE = #{absScore}, SEQUENCE = #{sequence}, SUB_SCORE = #{subScore} WHERE ID = #{id}")
    void update(AggregateQuadruplex quadruplex);

    // language=SQL
    @Delete("DELETE from AGGREGATE WHERE ID = #{id}")
    void delete(int id);

    // language=SQL
    @Insert("INSERT INTO AGGREGATE (ID, POSITION, LENGTH, SCORE, ABS_SCORE, SEQUENCE, SUB_SCORE) VALUES (#{id}, #{position}, #{length}, #{score}, #{absScore}, #{sequence}, #{subScore})")
    @Options(useGeneratedKeys = true, keyProperty = "id")
    void insert(AggregateQuadruplex quadruplex);

}
