package cz.mendelu.dnaAnalyser.analyse.palindrome.model;


import cz.mendelu.dnaAnalyser.sequence.NucleicBuffer;
import cz.mendelu.dnaAnalyser.sequence.data.SequenceData;
import cz.mendelu.dnaAnalyser.sequence.nucleotide.Nucleotide;
import lombok.Data;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Iterator;
import java.util.List;

/**
 * Created by Honza on 24. 7. 2015.
 */
public class PalindromeModel implements Iterable<PalindromeModel.Entry>/*, SubSequence*/ {

    public static final int SIZE_MARGIN = 5;

    private final SequenceData sequenceData;

    private final int position;
    private final int size;
    private final int spacer;
    private final int mismatches;

    public PalindromeModel(SequenceData sequenceData, int positions, int size, int spacer, int mismatches) {
        this.sequenceData = sequenceData;
        this.position = positions;
        this.size = size;
        this.spacer = spacer;
        this.mismatches = mismatches;
    }

    /*
        public Sequence getBefore() {
            int beforePosition = (position < SIZE_MARGIN) ? 0 : position - SIZE_MARGIN;
            int beforeSize = (position < SIZE_MARGIN) ? position : SIZE_MARGIN;
            return genom.getSequence(beforePosition, beforeSize);
        }

    */
    public NucleicBuffer getSequence() {
        return sequenceData.getBuffer(position, size);
    }


    public NucleicBuffer getSpacer() {
        return sequenceData.getBuffer(position + size, spacer);
    }


    public NucleicBuffer getOpposite() {
        return sequenceData.getBuffer(position + size + spacer, size);
    }
    /*
    public Sequence getAfter() {
        int afterPosition = position + size + spacer + size;
        int afterSize = (afterPosition > genom.getLength()) // Otestuji, zda pozice konce nalezeneho palindromu nekonci na zacatku genomu, jinymy slovy zda se nejedna o cyklu
                ? SIZE_MARGIN // Jde o cyklicky genom -> plna delka
                : (genom.getLength() > afterPosition + SIZE_MARGIN) // Nejde, pak vyberu zbytek do konce
                        ? SIZE_MARGIN // Tady nezasahu je do konce
                        : genom.getLength() - afterPosition; // Tady zasahuje a beru jen cast
        return genom.getSequence(afterPosition, afterSize);
    }
    */

    public float getMidlePosition() {
        return position + size + (spacer / 2f);
    }


    @Override
    public Iterator<Entry> iterator() {
        List<Entry> entries = new ArrayList<>();
        Iterator<Nucleotide> seq = getSequence();
        Iterator<Nucleotide> ops =  getOpposite().revert();
        // Nejprve vytvorim dvojice z sekvence a jejiho protejsku
        while (seq.hasNext() && ops.hasNext()) {
            entries.add(new Entry(seq.next(), ops.next()));
        }
        // Pote pridam spacer, ale bez protejsku
        getSpacer().forEachRemaining(nuclid -> entries.add(new Entry(nuclid, Nucleotide.NONE)));

        // Z vytvorene sekvence vradim iterator
        // + obalim zamcenym listem, abych zabranil odsranovani polozek
        return Collections.unmodifiableList(entries).iterator();
    }

    public Iterator<Entry> linearFormIterator() {
        List<Entry> entries = new ArrayList<>();
        getSequence().forEachRemaining(b -> entries.add(new Entry(b, b.supplement(sequenceData.getType()))));
        getSpacer().forEachRemaining(b -> entries.add(new Entry(b, b.supplement(sequenceData.getType()))));
        getOpposite().forEachRemaining(b -> entries.add(new Entry(b, b.supplement(sequenceData.getType()))));
        // Z vytvorene sekvence vradim iterator
        // + obalim zamcenym listem, abych zabranil odsranovani polozek
        return Collections.unmodifiableList(entries).iterator();
    }


    public long getMismatches() {
        return mismatches;
    }

    public int getPosition() {
        return position;
    }

    public int getDirection() {
        return 0;
    }

    public int getSize() {
        return size;
    }

    public int getLength() {
        return 2 * size + spacer;
    }

    @Data
    public static class Entry {

        private final Nucleotide nuclid;

        private final Nucleotide oposite;
    }
}
