package cz.mendelu.dnaAnalyser.analyse.zdna;

import cz.mendelu.dnaAnalyser.sequence.nucleotide.Nucleotide;
import cz.mendelu.dnaAnalyser.sequence.stream.Window;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.regex.Pattern;

@Slf4j
@Service
public class ZdnaAnalyser {
    // private static final int GCscore = 25;
    // private static final int nonGCscore = 3;
    private static float GCscore = 0;
    private static float GT_ACscore = 0;
    private static float ATscore = 0;

    private static float notPossibleZDNA = 0;

    /**
     * Find Z-DNAs for a given window
     *
     * @param minSequenceSize minimal length of searched sequences.
     * @param threshold minimal KV score of searched sequences.
     * @return arraylist of ZdnaAnalyseResults.
     */
    public ArrayList<ZdnaAnalyseResult> getResults(Window sequenceWindow, int minSequenceSize, float threshold, float gc, float gtac, float at, float oth) {
        GCscore = gc; // 1f;
        GT_ACscore = gtac;// 0.5f;
        ATscore = at; // 0.25f;
        notPossibleZDNA = oth; // 0;
        ArrayList<ZdnaAnalyseResult> allResults = new ArrayList<>();

        Pattern patternGC = Pattern.compile("(?=(GC|CG))");
        Pattern patternGT = Pattern.compile("(?=(GT|TG))");

        int i = 0;
        float score = 0;
        float tmpScore;
        int len = 1;

        while (i < (sequenceWindow.getSize())) {
            tmpScore = 0;
            if (i < (sequenceWindow.getSize() - 1)) {
                Nucleotide now = sequenceWindow.get(i);
                Nucleotide next = sequenceWindow.get(i + 1);

                tmpScore = getSubScore(now, next);
            }
            if (tmpScore > 0) {
                len++;
                score += tmpScore;
            } else {
                if (len >= minSequenceSize) {
                    float kvScore = score/2;
                    if (kvScore >= threshold) {
                        int start = i - len + 2;

                        StringBuilder seqB = new StringBuilder();
                        for (int k = 0; k < len; k++) {
                            seqB.append((char)sequenceWindow.get(start + k - 1).toByte());
                        }
                        String seq = seqB.toString();

                        int pcLen = len - 1;

                        // GC
                        double countGC = patternGC.matcher(seq).results().count();
                        double richnessGC = countGC / pcLen * 100;

                        // GT
                        double countGT = patternGT.matcher(seq).results().count();
                        double richnessGT = countGT / pcLen * 100;
                        float score10bp = kvScore / len * 10;

                        ZdnaAnalyseResult zdnaResult = new ZdnaAnalyseResult(
                                null, start, len, seq, richnessGC, richnessGT, kvScore, score10bp*10);

                        allResults.add(zdnaResult);
                    }
                }
                len = 1;
                score = 0;
            }
            i++;
        }

        return allResults;
    }

    private float getSubScore(Nucleotide first, Nucleotide second) {
        if ((first == Nucleotide.G && second == Nucleotide.C) || (first == Nucleotide.C && second == Nucleotide.G)) {
            return GCscore; // GC, CG
        } else if ((first == Nucleotide.G && second == Nucleotide.T) || (first == Nucleotide.T && second == Nucleotide.G) ||
                (first == Nucleotide.A && second == Nucleotide.C) || (first == Nucleotide.C && second == Nucleotide.A)) {
            return GT_ACscore; // GT, TG, AC, CA
        } else if ((first == Nucleotide.A && second == Nucleotide.T) || (first == Nucleotide.T && second == Nucleotide.A)) {
            return ATscore; // AT, TA
        } else {
            return notPossibleZDNA; // AA, AG, TT, TC, GA, GG, CT, CC
        }
    }


}
