package cz.mendelu.dnaAnalyser.analyse.zdna;

import cz.mendelu.dnaAnalyser.analyse.zdna.ZdnaAnalyseResult;
import cz.mendelu.dnaAnalyser.utils.analyse.data.DataLocalisedMapper;
import org.apache.ibatis.annotations.*;
import org.apache.ibatis.cursor.Cursor;
import org.apache.ibatis.session.RowBounds;

import java.util.ArrayList;
import java.util.List;

public interface ZdnaMapper extends DataLocalisedMapper<ZdnaAnalyseResult> {

    // language=SQL
    String TABLE = "CREATE TABLE IF NOT EXISTS ZDNA (" +
            "ID integer NOT NULL AUTO_INCREMENT, " +
            "POSITION integer, " +
            "LENGTH integer, " +
            "SEQUENCE varchar, " +
            "ZDNAGCRICHNESS varchar, " +
            "ZDNAGTRICHNESS varchar, " +
            "SCORE float, " +
            "SCORE10BP float," +
            "PRIMARY KEY (ID))";

    // language=SQL
    @Select("SELECT count(*) FROM ZDA")
    int countAll();

    // language=SQL
    @Select("SELECT * FROM ZDNA")
    @Results(value = {
            @Result(property = "id", column = "ID"),
            @Result(property = "position", column = "POSITION"),
            @Result(property = "length", column = "LENGTH"),
            @Result(property = "sequence", column = "SEQUENCE"),
            @Result(property = "zdnaGCRichness", column = "ZDNAGCRICHNESS"),
            @Result(property = "zdnaGTRichness", column = "ZDNAGTRICHNESS"),
            @Result(property = "score", column = "SCORE"),
            @Result(property = "score10bp", column = "SCORE10BP")
    })
    ArrayList<ZdnaAnalyseResult> getAll();

    // language=SQL
    @Select("SELECT * FROM ZDNA WHERE POSITION > #{start} AND POSITION < #{end}")
    @Results(value = {
            @Result(property = "id", column = "ID"),
            @Result(property = "position", column = "POSITION"),
            @Result(property = "length", column = "LENGTH"),
            @Result(property = "sequence", column = "SEQUENCE"),
            @Result(property = "zdnaGCRichness", column = "ZDNAGCRICHNESS"),
            @Result(property = "zdnaGTRichness", column = "ZDNAGTRICHNESS"),
            @Result(property = "score", column = "SCORE"),
            @Result(property = "score10bp", column = "SCORE10BP")
    })
    ArrayList<ZdnaAnalyseResult> getByWindow(Integer start, Integer end);

    // language=SQL
    @Select("SELECT * FROM ZDNA")
    @Results(value = {
            @Result(property = "id", column = "ID"),
            @Result(property = "position", column = "POSITION"),
            @Result(property = "length", column = "LENGTH"),
            @Result(property = "sequence", column = "SEQUENCE"),
            @Result(property = "zdnaGCRichness", column = "ZDNAGCRICHNESS"),
            @Result(property = "zdnaGTRichness", column = "ZDNAGTRICHNESS"),
            @Result(property = "score", column = "SCORE"),
            @Result(property = "score10bp", column = "SCORE10BP")
    })
    Cursor<ZdnaAnalyseResult> getCursor();

    // language=SQL
    @Select("SELECT * FROM ZDNA WHERE ID = #{id}")
    @Results(value = {
            @Result(property = "id", column = "ID"),
            @Result(property = "position", column = "POSITION"),
            @Result(property = "length", column = "LENGTH"),
            @Result(property = "sequence", column = "SEQUENCE"),
            @Result(property = "zdnaGCRichness", column = "ZDNAGCRICHNESS"),
            @Result(property = "zdnaGTRichness", column = "ZDNAGTRICHNESS"),
            @Result(property = "score", column = "SCORE"),
            @Result(property = "score10bp", column = "SCORE10BP")
    })
    ZdnaAnalyseResult getById(int id);

    // language=SQL
    @Update("UPDATE ZDNA SET " +
            "POSITION = #{position}, " +
            "LENGTH = #{length}, " +
            "SEQUENCE = #{sequence}, " +
            "ZDNAGCRICHNESS = #{zdnaGCRichness}, " +
            "ZDNAGTRICHNESS = #{zdnaGTRichness} " +
            "SCORE = #{score} " +
            "SCORE10BP = #{score10bp} " +
            "WHERE ID = #{id}")
    void update(ZdnaAnalyseResult result);

    // language=SQL
    @Delete("DELETE from ZDNA WHERE ID = #{id}")
    void delete(int id);

    // language=SQL
    @Insert("INSERT INTO ZDNA (ID, POSITION, LENGTH, SEQUENCE, ZDNAGCRICHNESS, ZDNAGTRICHNESS, SCORE, SCORE10BP)" +
            " VALUES " +
            "(#{id}, #{position}, #{length}, #{sequence}, #{zdnaGCRichness}, #{zdnaGTRichness}, #{score}, #{score10bp})"
    )
    @Options(useGeneratedKeys = true, keyProperty = "id")
    void insert(ZdnaAnalyseResult result);

    // language=SQL
    @Select("SELECT count(*) FROM ZDNA WHERE POSITION >= #{startPosition} AND POSITION < #{endPosition}")
    int countAllBetweenPosition(
            @Param("startPosition") int startPosition,
            @Param("endPosition") int endPosition);

    // language=SQL
    @Select("SELECT * FROM ZDNA WHERE POSITION >= #{startPosition} AND POSITION < #{endPosition} ORDER BY ${orderName} ASC")
    @Results(value = {
            @Result(property = "id", column = "ID"),
            @Result(property = "position", column = "POSITION"),
            @Result(property = "length", column = "LENGTH"),
            @Result(property = "sequence", column = "SEQUENCE"),
            @Result(property = "zdnaGCRichness", column = "ZDNAGCRICHNESS"),
            @Result(property = "zdnaGTRichness", column = "ZDNAGTRICHNESS"),
            @Result(property = "score", column = "SCORE"),
            @Result(property = "score10bp", column = "SCORE10BP")
    })
    List<ZdnaAnalyseResult> getAllBetweenPositionOrderAsc(
            @Param("startPosition") int startPosition,
            @Param("endPosition") int endPosition,
            @Param("orderName") String orderName,
            RowBounds rowBounds);

    // language=SQL
    @Select("SELECT * FROM ZDNA WHERE POSITION >= #{startPosition} AND POSITION < #{endPosition} ORDER BY ${orderName} DESC")
    @Results(value = {
            @Result(property = "id", column = "ID"),
            @Result(property = "position", column = "POSITION"),
            @Result(property = "length", column = "LENGTH"),
            @Result(property = "sequence", column = "SEQUENCE"),
            @Result(property = "zdnaGCRichness", column = "ZDNAGCRICHNESS"),
            @Result(property = "zdnaGTRichness", column = "ZDNAGTRICHNESS"),
            @Result(property = "score", column = "SCORE"),
            @Result(property = "score10bp", column = "SCORE10BP")
    })
    List<ZdnaAnalyseResult> getAllBetweenPositionOrderDesc(
            @Param("startPosition") int startPosition,
            @Param("endPosition") int endPosition,
            @Param("orderName") String orderName,
            RowBounds rowBounds);

}

