package cz.mendelu.dnaAnalyser.analyse.palindrome.api

import cz.mendelu.dnaAnalyser.analyse.palindrome.PalindromeAnalyseController
import cz.mendelu.dnaAnalyser.utils.api.ApiRestTemplate
import cz.mendelu.dnaAnalyser.utils.controller.PaginationParam
import cz.mendelu.dnaAnalyser.utils.controller.SortParam
import cz.mendelu.dnaAnalyser.utils.controller.TagModifyRequest
import org.springframework.beans.factory.annotation.Autowired
import org.springframework.boot.test.context.SpringBootTest
import org.springframework.http.HttpStatus
import org.springframework.test.context.ActiveProfiles
import spock.lang.Ignore
import spock.lang.Specification

import static org.springframework.boot.test.context.SpringBootTest.WebEnvironment.RANDOM_PORT

@SpringBootTest(webEnvironment = RANDOM_PORT)
@ActiveProfiles("test")
class PalindromeModelAnalyseControllerTest extends Specification {

    @Autowired
    ApiRestTemplate api

    def setup() {
        api.authentication('user@mendelu.cz', 'user')
    }


    @Ignore
    def "GET:/api/analyse/palindrome Get page with palindrome analysis"(){
        when:
        def response = api.get("/api/cruciform}")

        then:
        response.status == 200
    }

    @Ignore
    def "GET:/api/analyse/palindrome/tags Get all tags defined for sequences"(){
        setup:
        def paginationParam = new PaginationParam()
        def sortParam = new SortParam()

        when:
        def response = api.get("/api/analyse/palindrome/$sortParam/$paginationParam") //Tohle asi nebude OK

        then:
        response.status == 200
    }

    @Ignore
    def "GET:/api/analyse/palindrome/{id} Get one palindrome by ID"(){
        setup:
        def id = UUID.randomUUID()

        when:
        def response = api.get("/api/sequence/${id}")

        then:
        response.status == 200

    }

    @Ignore
    def "POST:/api/analyse/palindrome Create palindrome analyze."(){
        given:
        def request = new PalindromeAnalyseController.PalindromeAnalyseRequest(
                sequences: [UUID.fromString('987acc67-5714-47b1-b56f-1dc56ded7c87')]
        )

        when:
        def response = api.post("/api/analyse/palindrome", request)

        then:
        response.status == HttpStatus.CREATED
        response.body.size == 1
        and:
        def uuid = response.body.items[0].id
        response.body.batches[uuid] != null
    }

    @Ignore
    def "GET:/api/analyse/palindrome/{id}/tags Modify tags"(){
        setup:
        def id = UUID.fromString("987acc67-5714-47b1-b56f-1dc56ded7c87")
        def request = new TagModifyRequest(
                tags: ['new tags']
        )

        when:
        def response = api.put("/api/analyse/palindrome/${id}/tags", request)

        then:
        response.status == HttpStatus.ACCEPTED.value()

        and:
        def payload = response.body.payload
        payload.tags == ['new tags']
    }

    @Ignore
    def "DELETE:/api/analyse/palindrome/{id} Delete one palindrome analyze by ID"(){
        setup:
        def id = UUID.randomUUID()

        when:
        def response = api.delete("/api/analyse/palindrome/${id}")

        then:
        response.status == 204
    }

}
